@@ -45,7 +45,7 @@
 #include "asprintf.c"
 #endif
 
-#if !defined(asprintf)
+#if !defined(asprintf) && !defined(_FORTIFY_SOURCE)
 #include <stdarg.h>
 
 int asprintf(char **string_ptr, const char *format, ...)
@@ -260,7 +260,9 @@ static int checkFile(const char *fname)
 
 	/* Check if fname is ending in a taboo-extension; if so, return false */
 	for (i = 0; i < tabooCount; i++) {
-		asprintf(&pattern, "*%s", tabooExts[i]);
+		if (asprintf(&pattern, "*%s", tabooExts[i]) < 0) {
+			message(MESS_FATAL, "failed to allocate taboo pattern memory\n");
+		}
 		if (!fnmatch(pattern, fname, 0))
 		{
 			free(pattern);
@@ -695,6 +697,13 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
 
 	length = sb.st_size;
 
+	if (length > 0xffffff) {
+		message(MESS_ERROR, "file %s too large, probably not a config file.\n",
+				configFile);
+		close(fd);
+		return 1;
+	}   
+
 	/* We can't mmap empty file... */
 	if (length == 0) {
 		message(MESS_DEBUG,
@@ -803,7 +812,7 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
 					if (key == NULL)
 						continue;
 
-					rc = sscanf(key, "%200s %200s%c", createOwner,
+					rc = sscanf(key, "%199s %199s%c", createOwner,
 								createGroup, &foo);
 					if (rc == 3) {
 						message(MESS_ERROR, "%s:%d extra arguments for "
@@ -854,11 +863,11 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
 					if (key == NULL)
 						continue;
 
-					rc = sscanf(key, "%o %200s %200s%c", &createMode,
+					rc = sscanf(key, "%o %199s %199s%c", &createMode,
 							createOwner, createGroup, &foo);
 					/* We support 'create <owner> <group> notation now */
 					if (rc == 0) {
-						rc = sscanf(key, "%200s %200s%c",
+						rc = sscanf(key, "%199s %199s%c",
 								createOwner, createGroup, &foo);
 						/* Simulate that we have read createMode and se it
 						 * to NO_MODE. */
@@ -946,7 +955,7 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
 							multiplier = 1;
 						}
 
-						size = multiplier * strtoul(key, &chptr, 0);
+						size = multiplier * strtoull(key, &chptr, 0);
 						if (*chptr) {
 							message(MESS_ERROR, "%s:%d bad size '%s'\n",
 								configFile, lineNum, key);
@@ -984,6 +993,8 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
 						}
 					}
 					else continue;
+				} else if (!strcmp(key, "hourly")) {
+					newlog->criterium = ROT_HOURLY;
 				} else if (!strcmp(key, "daily")) {
 					newlog->criterium = ROT_DAYS;
 					newlog->threshhold = 1;
